#' Add a new report in a fairify project
#'
#' @param name Name of the report (sub-folder name in "reports")
#' @param template The report template chosen between the folders located in
#' the "templates" folder of the fairify project
#' @param lang Language used in the report as defined by the babel LaTeX package
#' @param path Path of the fairify project
#'
#' @return The path of the added report
#' @export
#'
#' @inherit create_reports examples
#'
add_report <- function(name,
                       template = "fairify:basic",
                       lang = "english",
                       path = pkgload::pkg_path()) {
  # reports folder handling
  if (!dir.exists(file.path(path, "reports"))) {
    message("Creating reports folder in: ", path, "...")
    create_reports(path)
  }
  # Checking template existence
  template_location <- get_template_location(template)

  # Creating report
  destpath <- file.path(path, "reports", name)
  if (dir.exists(destpath))
    stop("The folder ",
         destpath,
         " exists already. Delete it first or choose another.")
  dirtmp <- tempfile()
  dir.create(dirtmp, recursive = TRUE)
  b <- file.copy(
    from = file.path(pkg_sys("bookdown_template")),
    to = dirtmp,
    recursive = TRUE,
    copy.date = TRUE
  )
  if (any(!b)) stop("The creation of the report failed")
  file.rename(
    file.path(dirtmp, "bookdown_template"),
    destpath
  )

  bookdown_config <- list(
    language = list(ui = list(
      chapter_name = paste0(stringr::str_to_title(i18n("chapter", lang)), " ")
    )),
    babel_lang = lang
  )
  cat(
    yaml::as.yaml(bookdown_config),
    file = file.path(destpath, "_bookdown.yml"),
    append = TRUE
  )

  # Setting template in tex files
  lapply(c("in_header", "before_body", "after_body"), function(x) {
    tex_content <- sprintf("\\input{templates/%s/%s}", template_location[2], x)
    if (x == "in_header") {
      tex_content <- c(
        sprintf("\\usepackage[%s]{babel}", lang),
        sprintf("\\selectlanguage{%s}", lang),
        tex_content
      )
      babel_lang_tex <- sprintf("templates/basic/babel_%s.tex", lang)
      if (file.exists(pkg_sys(babel_lang_tex))) {
        tex_content <- c(tex_content, sprintf("\\input{%s}", babel_lang_tex))
      }
    }
    tex_content <- c(tex_content,
                     "\n% Add your own settings below to append or overwrite template settings")
    writeLines(tex_content, file.path(destpath, sprintf("%s.tex", x)))
  })
  # Setting template dependency
  writeLines(template, file.path(destpath, ".template_dependencies"))
  # Setting the setup R script
  cat(NULL, file = file.path(destpath, ".here"))
  writeLines(
    c(sprintf("source(here::here(\"templates/%s/setup.R\"))\n", template_location[2]),
      "# pkgload::load_all() # Uncomment to load current fairyfied project code and data\n",
      "# Add your own settings below to append or overwrite template settings"),
    file.path(destpath, "setup.R")
  )
  # Setting up the _output.yml file from templates
  write_output_yml(destpath)
  # Babel language support installation
  tinytex_install_babel_language_support(lang)

  return(destpath)
}

write_output_yml <- function(path) {
  template_dependencies <- get_template_dependencies(path)
  template_paths <- sapply(template_dependencies, function(x) {
    get_template_location(x)["path"]
  })
  output_paths <- file.path(template_paths, "_output.yml")

  output <- NULL
  for(output_path in output_paths) {
    if (file.exists(output_path)) {
      template_output <- yaml::read_yaml(output_path)
      output <- config::merge(output, template_output)
    }
  }
  yaml::write_yaml(output, file.path(path, "_output.yml"))
}

get_template_dependencies <- function(path) {
  dep_chain <- NULL
  i <- 0
  while(file.exists(file.path(path, ".template_dependencies"))) {
    dep_chain <- rbind(dep_chain,
                       data.frame(path = unname(path),
                                  template = readLines(
                                    file.path(path, ".template_dependencies")
                                  )))
    i <- i + 1
    path <- get_template_location(dep_chain$template[i])["path"]
  }
  if (!is.null(dep_chain)) {
    return(unique(rev(dep_chain$template)))
  } else {
    return(NULL)
  }
}
