#' Generate a file from a mermaid diagram
#'
#' This function download the file from https://mermaid.ink which generates the image.
#' The file is downloaded only if it does not already exist.
#'
#' @details
#' Use this function with [knitr::include_graphics] to display a mermaid diagram
#' in a Rmarkdown document. Compared to the `diagrammeR::mermaid` function, the
#' generated image is not a HTMLwidget and can be knit in pdf through latex and
#' moreover, its size can be controlled with `fig.width` and `fig.height`.
#'
#' @param diagram Diagram in mermaid markdown-like language or file (as a connection or file name) containing a diagram specification
#' @param theme Mermaid theme (See https://mermaid.js.org/config/theming.html#available-themes)
#' @param format Image format (either `"jpg"`, or `"png"`, or `"svg"`)
#' @param dir.dest Destination folder for the downloaded image. This parameter is
#' ignored if `file.dest` contains a folder path.
#' @param file.dest Path to the downloaded image. It's combined with `dir.dest`
#' if it only contains the name of the file without a folder path.
#' @param link Link generated by [mermaid_gen_link]
#' @param server URL of the server used to generate the link
#'
#' @return The path to the downloaded image.
#' @export
#' @rdname mermaid
#'
#' @examples
#' diagram <- "flowchart LR\n  A --> B"
#' mermaid_gen_link(diagram)
#' f <- mermaid(diagram)
#' f
#' \dontrun{
#' # For displaying the diagram in Rmarkdown document
#' knitr::include_graphics(mermaid(diagram))
#' }
#'
#' # Clean temporary folder
#' unlink(f)
mermaid <- function(diagram,
                    format = "png",
                    theme = "default",
                    dir.dest = tempdir(),
                    file.dest = paste0(rlang::hash(link), ".", format),
                    link = mermaid_gen_link(diagram, theme = theme, format = format)) {
  if (!dir.exists(dir.dest)) dir.create(dir.dest, recursive = TRUE, showWarnings = FALSE)
  if (dirname(file.dest) == ".") file.dest <- file.path(dir.dest, file.dest)
  if (!file.exists(file.dest)) {
    download.file(link, file.dest, quiet = TRUE, mode = "wb")
  }
  return(file.dest)
}

#' Compress data in pako format
#'
#' @param data [character] data to compress
#'
#' @return Raw compresses data.
#' @source Translated from python script
#' https://github.com/mermaid-js/mermaid-live-editor/discussions/1291#discussioncomment-6837936
#' @noRd
pako_deflate <- function(data) {
  # compress = zlib.compressobj(9, zlib.DEFLATED, 15, 8,zlib.Z_DEFAULT_STRATEGY)
  compress <- zlib::compressobj(
    level = 9,
    method = zlib::zlib$DEFLATED,
    wbits = 15,
    memLevel = 8,
    strategy = zlib::zlib$Z_DEFAULT_STRATEGY
  )
  compressed_data <- compress$compress(charToRaw(data))
  compressed_data <- c(compressed_data, compress$flush())
return(compressed_data)
}

#' @rdname mermaid
#' @export
mermaid_gen_link <- function(diagram, theme = "default", format = "png", server = "https://mermaid.ink") {
  is_connection_or_file <- inherits(diagram[1], "connection") ||
    file.exists(diagram[1])
  if (is_connection_or_file) {
    diagram <- readLines(diagram, encoding = "UTF-8", warn = FALSE)
  }
  if (length(diagram) > 1) {
    diagram <- paste(diagram, collapse = "\n")
  }
  jGraph <-
    list(code = diagram,
         mermaid = list(theme = theme)) |> jsonlite::toJSON(auto_unbox = TRUE)
  deflated <- pako_deflate(jGraph)
  dEncode = gsub("\n", "", jsonlite::base64url_enc(deflated))
  mode <- ifelse(format != "svg", "img", "svg")
  link = sprintf("%s/%s/pako:%s", server, mode, dEncode)
  if (format != "svg") {
    link <- paste0(link, "?type=", format)
  }
  return(link)
}
